package cn.aibreeze.exam.api;

import cn.aibreeze.exam.dto.ApiResponse;
import cn.aibreeze.exam.dto.GetQuestionResponseResult;
import cn.aibreeze.exam.dto.UpdateQuestionRequestDto;
import cn.aibreeze.exam.entity.ElehKnowledgeDo;
import cn.aibreeze.exam.entity.ElehKnowledgeTagDo;
import cn.aibreeze.exam.entity.ElehSessionQaRecordDo;
import cn.aibreeze.exam.mapper.ElehKnowledgeMapper;
import cn.aibreeze.exam.mapper.ElehKnowledgeTagMapper;
import cn.aibreeze.exam.mapper.ElehSessionQaRecordMapper;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Objects;


@RestController
@RequestMapping(value = "/exam")
@RequiredArgsConstructor
public class ExamController {


    private final ElehKnowledgeMapper knowledgeMapper;

    private final ElehKnowledgeTagMapper tagMapper;

    private final ElehSessionQaRecordMapper qaRecordMapper;

    @GetMapping(value = "/list")
    public String list() {
        LambdaQueryWrapper<ElehKnowledgeDo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ElehKnowledgeDo::getTenant, 16);
        List<ElehKnowledgeDo> elehKnowledgeDos = knowledgeMapper.selectList(queryWrapper);
        return JSONObject.toJSONString(elehKnowledgeDos);
    }

    @GetMapping(value = "/get_question")
    public ApiResponse<GetQuestionResponseResult> getQuestion(@RequestParam("tenant") Integer tenant,
                                                              @RequestParam("code") String code,
                                                              @RequestParam("session_id") String sessionId,
                                                              @RequestParam("user_id") String userId) {

        GetQuestionResponseResult result = new GetQuestionResponseResult();

        // 根据分类获取考题
        ElehKnowledgeTagDo tagDo = tagMapper.selectByCode(tenant, code);
        if(Objects.nonNull(tagDo)) {
            Long maxQuestionId = qaRecordMapper.selectSessionMaxQuestionId(tenant, sessionId, userId);
            maxQuestionId = Objects.nonNull(maxQuestionId) ? maxQuestionId : 0;
            ElehKnowledgeDo knowledgeDo = knowledgeMapper.selectQuestion(tenant, tagDo.getId(), maxQuestionId);
            if(Objects.nonNull(knowledgeDo)) {
                result.setQuestion(knowledgeDo.getQuestion());
                result.setAnswer(knowledgeDo.getAnswer());
                result.setProject(tagDo.getName());
                result.setCode(tagDo.getCode());
                result.setQuestionId(knowledgeDo.getId());
                result.setNext(true);
            }
        }

        return ApiResponse.ok(result);
    }

    @PostMapping(value = "update_question")
    public ApiResponse<String> updateQuestion(@RequestBody UpdateQuestionRequestDto requestDto) {

        String answer = requestDto.getAnswer();
        Integer tenant = requestDto.getTenant();
        String sessionId = requestDto.getSessionId();
        String userId = requestDto.getUserId();
        Long questionId = requestDto.getQuestionId();


        ElehSessionQaRecordDo recordDo = new ElehSessionQaRecordDo();
        recordDo.setQuestionId(questionId);
        recordDo.setAnswer(answer);
        recordDo.setTenant(tenant);
        recordDo.setSessionId(sessionId);
        recordDo.setUserId(userId);
        recordDo.setScoreCriteria(requestDto.getScoreCriteria());
        recordDo.setScore(recordDo.getScore());
        recordDo.setCreatedAt(new Date());
        qaRecordMapper.insert(recordDo);

        return ApiResponse.ok("SUCCESS");
    }

    @GetMapping(value = "/get_final_result")
    public ApiResponse<Double> getFinalResult(@RequestParam("tenant") Integer tenant,
                                 @RequestParam("session_id") String sessionId,
                                 @RequestParam("user_id") String userId) {

        Double avgScore = 0D;

        LambdaQueryWrapper<ElehSessionQaRecordDo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ElehSessionQaRecordDo::getTenant, tenant);
        queryWrapper.eq(ElehSessionQaRecordDo::getSessionId, sessionId);
        queryWrapper.eq(ElehSessionQaRecordDo::getUserId, userId);
        List<ElehSessionQaRecordDo> qaRecordDos = qaRecordMapper.selectList(queryWrapper);
        if(!CollectionUtils.isEmpty(qaRecordDos)) {
            double sum = qaRecordDos.stream().mapToDouble(ElehSessionQaRecordDo::getScore).sum();
            avgScore = sum / qaRecordDos.size();
        }
        return ApiResponse.ok(avgScore);
    }

}
